﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class WavesManager : MonoBehaviour
{
    [SerializeField] AudioSource startSound;
    [SerializeField] AudioSource nextSound;
    [SerializeField] AudioSource music;
    [SerializeField] AudioSource bossMusic;
    [SerializeField] AudioSource bossSound;

    [SerializeField] Text waveIndicator;

    [SerializeField] Enemy enemy01;
    [SerializeField] Enemy enemy02;
    [SerializeField] Enemy enemy03;
    [SerializeField] Enemy enemy04;
    [SerializeField] Enemy enemy05;

    float startX = -11f;
    GameManager gameManager;

    int waveCount = 0;

    // Use this for initialization
    void Start()
    {
        music.Play();
        gameManager = GameObject.Find("Game Manager").GetComponent<GameManager>();
        StartCoroutine(Wave01());
    }

    IEnumerator WaveStart(int enemyCount)
    {
        waveCount++;

        if(waveCount == 1)
            startSound.Play();
        else
            nextSound.Play();

        waveIndicator.text = "WAVE " + waveCount.ToString("00");
        yield return new WaitForSeconds(1);
        for (int i = 1; i <= 10; i++)
        {
            yield return new WaitForSeconds(0.1f);
            waveIndicator.color = new Color(waveIndicator.color.r, waveIndicator.color.g, waveIndicator.color.b, i * 0.1f);
        }
        yield return new WaitForSeconds(1);
        gameManager.NextWave(enemyCount);

        for (int i = 1; i <= 5; i++)
        {
            yield return new WaitForSeconds(0.1f);
            waveIndicator.color = new Color(waveIndicator.color.r, waveIndicator.color.g, waveIndicator.color.b, 1 - (i * 0.2f));
        }
    }

    Enemy Instantiate(Enemy prefab, Vector2 position)
    {
        return Instantiate(prefab, position, Quaternion.identity);
    }

    IEnumerator Wave01()
    {
        yield return new WaitForSeconds(3);
        yield return WaveStart(6);
        Instantiate(enemy01, new Vector2(startX, 0));
        Instantiate(enemy01, new Vector2(startX - 0.5f, 2));
        Instantiate(enemy01, new Vector2(startX - 0.5f, -2));

        yield return new WaitForSeconds(4);

        Instantiate(enemy02, new Vector2(startX, 0));
        Instantiate(enemy02, new Vector2(startX, -1));
        Instantiate(enemy02, new Vector2(startX, 1));

        yield return new WaitUntil(() => gameManager.enemy == 0);
        yield return Wave02();
    }

    IEnumerator Wave02()
    {
        yield return WaveStart(13);

        Instantiate(enemy01, new Vector2(startX, 0));
        Instantiate(enemy01, new Vector2(startX, -2));
        Instantiate(enemy01, new Vector2(startX, 2));
        Instantiate(enemy02, new Vector2(startX-1, -1));
        Instantiate(enemy02, new Vector2(startX-1, 1));

        yield return new WaitForSeconds(3);

        for (int i = 0; i < 8; i++)
            Instantiate(enemy01, new Vector2(startX - i, 2 - i));

        yield return new WaitUntil(() => gameManager.enemy == 0);
        yield return Wave03();
    }
    IEnumerator Wave03()
    {
        yield return WaveStart(8);

        Instantiate(enemy03, new Vector2(startX, -2));
        Instantiate(enemy03, new Vector2(startX, -3.5f));

        yield return new WaitForSeconds(6);

        Instantiate(enemy03, new Vector2(startX, 3));
        Instantiate(enemy03, new Vector2(startX, 1.5f));
        Instantiate(enemy03, new Vector2(startX, 0));
        Instantiate(enemy01, new Vector2(startX-1, 3));
        Instantiate(enemy01, new Vector2(startX-1, 1.5f));
        Instantiate(enemy01, new Vector2(startX-1, 0));

        yield return new WaitUntil(() => gameManager.enemy == 0);
        yield return Wave04();
    }
    IEnumerator Wave04()
    {
        yield return WaveStart(19);

        Instantiate(enemy03, new Vector2(startX, 3));
        Instantiate(enemy03, new Vector2(startX, 1.5f));
        Instantiate(enemy03, new Vector2(startX, 0));
        Instantiate(enemy02, new Vector2(startX - 1, 3));
        Instantiate(enemy02, new Vector2(startX - 1, 1.5f));
        Instantiate(enemy02, new Vector2(startX - 1, 0));

        yield return new WaitForSeconds(5);

        Instantiate(enemy03, new Vector2(startX, 0));
        Instantiate(enemy03, new Vector2(startX, -1.5f));
        Instantiate(enemy03, new Vector2(startX, -3));
        Instantiate(enemy01, new Vector2(startX - 1, 0));
        Instantiate(enemy01, new Vector2(startX - 1, -1.5f));
        Instantiate(enemy01, new Vector2(startX - 1, -3));

        yield return new WaitForSeconds(5);

        Instantiate(enemy03, new Vector2(startX, -2.5f));
        Instantiate(enemy03, new Vector2(startX, 2.5f));
        Instantiate(enemy03, new Vector2(startX, 0));
        Instantiate(enemy02, new Vector2(startX - 1, -1.25f));
        Instantiate(enemy02, new Vector2(startX - 1, 1.25f));
        Instantiate(enemy02, new Vector2(startX - 1, -2.75f));
        Instantiate(enemy02, new Vector2(startX - 1, 2.75f));


        yield return new WaitUntil(() => gameManager.enemy == 0);
        yield return Wave05();
    }

    IEnumerator Wave05()
    {
        yield return WaveStart(13);

        Instantiate(enemy04, new Vector2(startX, 0));
        yield return new WaitForSeconds(2);
        Instantiate(enemy04, new Vector2(startX, 2));
        yield return new WaitForSeconds(2);
        Instantiate(enemy04, new Vector2(startX, -2));
        yield return new WaitForSeconds(1);
        for (int i = 0; i < 10; i++)
        {
            Instantiate(enemy04, new Vector2(startX, 1));
            yield return new WaitForSeconds(0.2f);
        }

        yield return new WaitUntil(() => gameManager.enemy == 0);
        yield return Wave06();
    }

    IEnumerator Wave06()
    {
        yield return WaveStart(27);

        for (float y = -5; y <= 5; y += 1.2f)
        {
            Instantiate(enemy03, new Vector2(startX, y));
            Instantiate(enemy01, new Vector2(startX - 1, y));
            Instantiate(enemy01, new Vector2(startX - 2, y));
        }

        yield return new WaitUntil(() => gameManager.enemy == 0);
        yield return Wave07();
    }

    IEnumerator Wave07()
    {
        yield return WaveStart(25);

        for(float y = -2f; y <= 2; y+= 1)
        {
            for(float x = 0; x < 5; x += 1)
            {
                if(y == 2)
                    Instantiate(enemy03, new Vector2(startX - x, y));
                else if (x%2 == 0)
                    Instantiate(enemy02, new Vector2(startX - x, y));
                else
                    Instantiate(enemy01, new Vector2(startX - x, y));
            }
        }

        yield return new WaitUntil(() => gameManager.enemy == 0);
        yield return Wave10();
    }

    IEnumerator Wave08()
    {
        yield return WaveStart(0);

        yield return new WaitUntil(() => gameManager.enemy == 0);
        yield return Wave09();
    }
    IEnumerator Wave09()
    {
        yield return WaveStart(0);

        yield return new WaitUntil(() => gameManager.enemy == 0);
        yield return Wave10();
    }
    IEnumerator Wave10()
    {
        for (float v = 1; v >= 0; v -= 0.05f)
        {
            yield return new WaitForSeconds(0.1f);
            music.volume = v;
        }
        music.Stop();
        bossMusic.Play();
        bossSound.Play();

         yield return WaveStart(1);

        Enemy boss = Instantiate(enemy05, new Vector2(startX, 0));
        /*
        Instantiate(enemy03, new Vector2(startX, -2));
        Instantiate(enemy03, new Vector2(startX, -1));
        Instantiate(enemy03, new Vector2(startX, 0));
        Instantiate(enemy03, new Vector2(startX, 1));
        Instantiate(enemy03, new Vector2(startX, 2));
        while (boss != null)
        {
            Instantiate(enemy01, new Vector2(startX, -3.5f));
            Instantiate(enemy01, new Vector2(startX, 3.5f));
            yield return new WaitForSeconds(2f);
            Instantiate(enemy02, new Vector2(startX, -2f));
            Instantiate(enemy02, new Vector2(startX, 2));
            yield return new WaitForSeconds(2f);
            Instantiate(enemy01, new Vector2(startX, -3.5f));
            Instantiate(enemy01, new Vector2(startX, 3.5f));
            yield return new WaitForSeconds(2f);
            Instantiate(enemy02, new Vector2(startX, -5f));
            Instantiate(enemy02, new Vector2(startX, 5f));
            yield return new WaitForSeconds(2f);
        }
        */
        yield return new WaitUntil(() => gameManager.enemy == 0);
        gameManager.Victory();
        bossSound.Stop();


        for (float v = 1; v >= 0; v -= 0.05f)
        {
            yield return new WaitForSeconds(0.1f);
            bossMusic.volume = v;
        }
        //yield return WaveXX();
    }


    IEnumerator WaveXX()
    {
        yield return WaveStart(0);

        yield return new WaitUntil(() => gameManager.enemy == 0);
        yield return WaveXX();
    }

        /*
        public int GetEnnemyCount(List<int> randoms)
        {
            int result = 0;
            for (int i = 0; i < randoms.Count; i += 2)
            {

                if (randoms[i] < 4)
                {
                    result += 2;
                }
                else if (randoms[i] < 6)
                {
                    switch (randoms[i + 1])
                    {
                        case 0:
                            result += 6;
                            break;
                        case 1:
                            result += 6;
                            break;
                        case 2:
                            result += 5;
                            break;
                    }
                }
                else if (randoms[i] < 10)
                {
                    switch (randoms[i + 1])
                    {
                        case 0:
                            result += 9;
                            break;
                        case 1:
                            result += 18;
                            break;
                        case 2:
                            result += 18;
                            break;
                    }
                }
                else if (randoms[i] < 20)
                {
                    switch (randoms[i + 1])
                    {
                        case 0:
                            result += 0;
                            break;
                        case 1:
                            result += 0;
                            break;
                        case 2:
                            result += 0;
                            break;
                    }
                }
                else
                {
                    switch (randoms[i + 1])
                    {
                        case 0:
                            result += 0;
                            break;
                        case 1:
                            result += 0;
                            break;
                        case 2:
                            result += 0;
                            break;
                    }
                }
            }
            return result;
        }


        IEnumerator WavesGenerator()
        {
            while (true)
            {
                List<int> randoms = new List<int>();
                for (int i = 0; i < waveCount; i++)
                {
                    randoms.Add(waveCount);
                    randoms.Add(Random.Range(0, 3));
                }
                enemyCount = GetEnnemyCount(randoms);

                yield return WaveStart(waveCount);

                for (int i = 0; i < randoms.Count; i += 2)
                {
                    yield return GenerateWave(randoms[i], randoms[i + 1]);
                }

                yield return new WaitUntil(() => gameManager.enemy == 0);
                waveCount++;
            }
        }

        IEnumerator GenerateWave(int difficulty, int waveID)
        {
            if (difficulty < 4)
            {
                yield return StartWave(waveID);
            }
            else if (difficulty < 6)
            {
                yield return EasyWave(waveID);
            }
            else if (difficulty < 11)
            {
                yield return MediumWave(waveID);
            }
            else if (difficulty < 21)
            {
                yield return HardWave(waveID);
            }
            else
            {
                yield return VeryHardWave(waveID);
            }
        }

        IEnumerator StartWave(int waveID)
        {
            switch (waveID)
            {
                case 0:
                    yield return Smallest01();
                    break;
                case 1:
                    yield return Smallest02();
                    break;
                case 2:
                    yield return Smallest03();
                    break;
            }
            yield return new WaitForSeconds(Random.Range(3, 5));
        }
        IEnumerator EasyWave(int waveID)
        {
            switch (waveID)
            {
                case 0:
                    yield return Easy00();
                    break;
                case 1:
                    yield return Easy01();
                    break;
                case 2:
                    yield return Easy02();
                    break;
            }
            yield return new WaitForSeconds(Random.Range(5, 10));
        }

        IEnumerator MediumWave(int waveID)
        {
            switch (waveID)
            {
                case 0:
                    yield return Medium00();
                    break;
                case 1:
                    yield return Medium01();
                    break;
                case 2:
                    yield return Medium02();
                    break;
            }
            yield return new WaitForSeconds(Random.Range(3, 6));
        }
        IEnumerator HardWave(int waveID)
        {
            switch (waveID)
            {
                case 0:
                    yield return Hard00();
                    break;
                case 1:
                    yield return Hard01();
                    break;
                case 2:
                    yield return Hard02();
                    break;
            }
            yield return new WaitForSeconds(Random.Range(10, 15));

        }
        IEnumerator VeryHardWave(int waveID)
        {
            switch (waveID)
            {
                case 0:
                    yield return VeryHard00();
                    break;
                case 1:
                    yield return VeryHard01();
                    break;
                case 2:
                    yield return VeryHard02();
                    break;
            }
            yield return new WaitForSeconds(Random.Range(10, 15));
        }


        void EnemySet01(Vector2 center)
        {
            Instantiate(enemy01, new Vector3(center.x, center.y - 1, 0), Quaternion.identity);
            Instantiate(enemy01, new Vector3(center.x, center.y + 1, 0), Quaternion.identity);
        }
        void EnemySet02(Vector2 center)
        {
            Instantiate(enemy02, new Vector3(center.x, center.y - 1, 0), Quaternion.identity);
            Instantiate(enemy02, new Vector3(center.x, center.y + 1, 0), Quaternion.identity);
        }
        void EnemySet03(Vector2 center)
        {
            Instantiate(enemy03, new Vector3(center.x, center.y - 1, 0), Quaternion.identity);
            Instantiate(enemy03, new Vector3(center.x, center.y + 1, 0), Quaternion.identity);
        }

        void EnemySet04(Vector2 center)
        {
            Instantiate(enemy03, new Vector3(center.x, center.y, 0), Quaternion.identity);
            Instantiate(enemy03, new Vector3(center.x, center.y + 1.5f, 0), Quaternion.identity);
            Instantiate(enemy03, new Vector3(center.x, center.y - 1.5f, 0), Quaternion.identity);
            Instantiate(enemy01, new Vector3(center.x - 1, center.y, 0), Quaternion.identity);
            Instantiate(enemy01, new Vector3(center.x - 1, center.y + 1.5f, 0), Quaternion.identity);
            Instantiate(enemy01, new Vector3(center.x - 1, center.y - 1.5f, 0), Quaternion.identity);
        }
        void EnemySet05(Vector2 center)
        {
            Instantiate(enemy03, new Vector3(center.x, center.y, 0), Quaternion.identity);
            Instantiate(enemy03, new Vector3(center.x, center.y + 1.5f, 0), Quaternion.identity);
            Instantiate(enemy03, new Vector3(center.x, center.y - 1.5f, 0), Quaternion.identity);
            Instantiate(enemy02, new Vector3(center.x - 1, center.y, 0), Quaternion.identity);
            Instantiate(enemy02, new Vector3(center.x - 1, center.y + 1.5f, 0), Quaternion.identity);
            Instantiate(enemy02, new Vector3(center.x - 1, center.y - 1.5f, 0), Quaternion.identity);
        }

        void EnemySet06(Vector2 center)
        {
            Instantiate(enemy01, new Vector3(center.x, center.y, 0), Quaternion.identity);
            Instantiate(enemy01, new Vector3(center.x - 1, center.y - 1, 0), Quaternion.identity);
            Instantiate(enemy01, new Vector3(center.x - 1, center.y + 1, 0), Quaternion.identity);
            Instantiate(enemy01, new Vector3(center.x - 2, center.y - 2, 0), Quaternion.identity);
            Instantiate(enemy01, new Vector3(center.x - 2, center.y + 2, 0), Quaternion.identity);
        }

        void EnemySet07(Vector2 center)
        {
            Instantiate(enemy04, new Vector3(center.x, center.y + 1, 0), Quaternion.identity);
            Instantiate(enemy04, new Vector3(center.x, center.y, 0), Quaternion.identity);
            Instantiate(enemy04, new Vector3(center.x, center.y - 1, 0), Quaternion.identity);
            Instantiate(enemy04, new Vector3(center.x - 1, center.y - 1, 0), Quaternion.identity);
            Instantiate(enemy04, new Vector3(center.x - 1, center.y, 0), Quaternion.identity);
            Instantiate(enemy04, new Vector3(center.x - 1, center.y + 1, 0), Quaternion.identity);
            Instantiate(enemy04, new Vector3(center.x - 2, center.y - 1, 0), Quaternion.identity);
            Instantiate(enemy04, new Vector3(center.x - 2, center.y , 0), Quaternion.identity);
            Instantiate(enemy04, new Vector3(center.x - 2, center.y + 1, 0), Quaternion.identity);
        }

        IEnumerator Smallest01()
        {
            float random = Random.Range(-3f, 3f);
            EnemySet01(new Vector2(startX, random));

            yield return new WaitForSeconds(1);
        }
        IEnumerator Smallest02()
        {
            float random = Random.Range(-3f, 3f);
            EnemySet02(new Vector2(startX, random));
            yield return new WaitForSeconds(1);
        }
        IEnumerator Smallest03()
        {
            float random = Random.Range(-3f, 3f);
            EnemySet03(new Vector2(startX, random));
            yield return new WaitForSeconds(1);
        }

        IEnumerator Easy00()
        {
            float random = Random.Range(-3f, 3f);
            EnemySet04(new Vector2(startX, random));
            yield return new WaitForSeconds(1);
        }

        IEnumerator Easy01()
        {
            float random = Random.Range(-3f, 3f);
            EnemySet05(new Vector2(startX, random));
            yield return new WaitForSeconds(1);
        }
        IEnumerator Easy02()
        {
            float random = Random.Range(-3f, 3f);
            EnemySet06(new Vector2(startX, random));
            yield return new WaitForSeconds(1);
        }


        IEnumerator Medium00()
        {
            float random = Random.Range(0f, 3f);
            EnemySet07(new Vector2(startX, random));

            yield return new WaitForSeconds(1);
        }

        IEnumerator Medium01()
        {
            float random = Random.Range(-0f, 3f);
            EnemySet02(new Vector2(startX, random));
            random = Random.Range(-3f, 0f);
            EnemySet02(new Vector2(startX, random));
            yield return new WaitForSeconds(5);

            random = Random.Range(0f, 3f);
            EnemySet05(new Vector2(startX, random));
            yield return new WaitForSeconds(5);

            random = Random.Range(-0f, 3f);
            EnemySet01(new Vector2(startX, random));
            random = Random.Range(-3f, 0f);
            EnemySet02(new Vector2(startX, random));
            yield return new WaitForSeconds(5);
        }
        IEnumerator Medium02()
        {
            float random = Random.Range(-0f, 3f);
            EnemySet02(new Vector2(startX, random));
            random = Random.Range(-3f, 0f);
            EnemySet02(new Vector2(startX, random));
            yield return new WaitForSeconds(5);

            random = Random.Range(-2f, 2f);
            EnemySet05(new Vector2(startX, random));
            yield return new WaitForSeconds(5);

            random = Random.Range(-3f, 0f);
            EnemySet01(new Vector2(startX, random));
            yield return new WaitForSeconds(1);
            random = Random.Range(0f, 3f);
            EnemySet02(new Vector2(startX, random));
            yield return new WaitForSeconds(1);
        }


        IEnumerator Hard00()
        {
            yield return new WaitForSeconds(1);
        }

        IEnumerator Hard01()
        {
            yield return new WaitForSeconds(2);
        }
        IEnumerator Hard02()
        {
            yield return new WaitForSeconds(2);
        }
        IEnumerator VeryHard00()
        {
            yield return new WaitForSeconds(1);
        }

        IEnumerator VeryHard01()
        {
            yield return new WaitForSeconds(1);
        }
        IEnumerator VeryHard02()
        {
            yield return new WaitForSeconds(1);
        }
        */
    }
