﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class GameManager : MonoBehaviour {
    
    [SerializeField] Image hidePanel;
    [SerializeField] Image credits;
    public GameObject explosion;
    public GameObject bigExplosion;
    public AudioSource explosionSound;

    [SerializeField] AudioSource music;
    [SerializeField] AudioSource victoryMusic;
    [SerializeField] AudioSource selectSound;
    [SerializeField] AudioSource openMenuSound;
    [SerializeField] NewHighScorePanel newScorePanel;
    [SerializeField] GameOverPanel gameOverPanel;
    [SerializeField] GameObject menu;
    [SerializeField] Score scoreUp;

    [SerializeField] Text scoreHUD;
    public int score;
    [SerializeField] Text enemyHUD;
    public int enemy;
    [SerializeField] Text waveHUD;
    public int wave;

    public static bool gamePaused = false;
    
    [SerializeField] GameObject[] playerIndicators;

    [SerializeField] Button[] menuButtons;
    [SerializeField] GameObject cursor;
    int selectedMenu = 0;
    bool gamePadUsed = false;

    // Use this for initialization
    void Start () {
        gamePaused = true;
        StartCoroutine(StartGame());
	}

    IEnumerator StartGame()
    {
        score = 0;
        enemy = 0;
        wave = 0;
        UpdateTexts();

        hidePanel.gameObject.SetActive(true);
        for (float i = 0; i < 30; i += 1)
        {
            hidePanel.GetComponent<Image>().color = new Color(0, 0, 0, 1 - i / 30);
            yield return new WaitForSeconds(0.002f);
        }
        hidePanel.gameObject.SetActive(false);

        yield return new WaitForSeconds(0.5f);

        playerIndicators[0].SetActive(false);
        playerIndicators[1].SetActive(false);
        gamePaused = false;
    }

    void ChangeMenu(int idx)
    {
        selectSound.Play();
        if (idx < 0) idx = 0;
        if (idx >= menuButtons.Length) idx = menuButtons.Length - 1;
        menuButtons[selectedMenu].GetComponentInChildren<Text>().color = new Color(199f / 255, 196f / 255, 182f / 255);
        selectedMenu = idx;
        menuButtons[selectedMenu].GetComponentInChildren<Text>().color = new Color(131f / 255, 255f / 255, 230f / 255);
        cursor.transform.position = new Vector3(cursor.transform.position.x, menuButtons[selectedMenu].transform.position.y, 0);
    }

    // Update is called once per frame
    void Update () {
        if (menu.activeSelf)
        {
            if (Input.GetKeyDown(KeyCode.S) || Input.GetKeyDown(KeyCode.DownArrow) )
            {
                ChangeMenu(selectedMenu + 1);
            }
            else if (Input.GetKeyDown(KeyCode.W) || Input.GetKeyDown(KeyCode.UpArrow))
            {
                ChangeMenu(selectedMenu - 1);
            }
            else if (Input.GetButtonDown("Action P1") || Input.GetButtonDown("Action P2"))
            {
                ActiveSelectedMenu();
            }

            if (Input.GetAxis("Vertical P1") < 0 && !gamePadUsed)
            {
                ChangeMenu(selectedMenu + 1);
                gamePadUsed = true;
            }
            else if (Input.GetAxis("Vertical P1") > 0 && !gamePadUsed)
            {
                ChangeMenu(selectedMenu - 1);
                gamePadUsed = true;
            }
            else if (Input.GetAxis("Vertical P1") == 0)
            {
                gamePadUsed = false;
            }
        }

        if(credits.gameObject.activeSelf && Input.anyKey)
        {
            StartCoroutine(CheckForPoints());
        }

        BackToMainMenu();

        if (Input.GetButtonDown("Start"))
        {
            OpenCloseMenu();
        }
    }
    void ActiveSelectedMenu()
    {
        switch (selectedMenu)
        {
            case 0:
                OpenCloseMenu();
                break;
            case 1:
                //TODO;
                break;
            case 2:
                Quit();
                break;
        }
    }

    void BackToMainMenu()
    {
        if (gameOverPanel.gameObject.activeSelf)
        {
            if (Input.GetKeyDown(KeyCode.Mouse0))
            {
                StartCoroutine(BackToMenuAnim());
            }
        }
    }

    public IEnumerator BackToMenuAnim()
    {
        hidePanel.gameObject.SetActive(true);
        for (float i = 0; i < 30; i += 1)
        {
            hidePanel.GetComponent<Image>().color = new Color(0, 0, 0, i / 30);
            yield return new WaitForSeconds(0.002f);
        }
        UnityEngine.SceneManagement.SceneManager.LoadScene("Menu");
    }

    public void Victory()
    {
        StartCoroutine(VictoryAnim());
    }

    IEnumerator VictoryAnim()
    {
        victoryMusic.Play();
        gamePaused = true;
        hidePanel.gameObject.SetActive(true);
        for (float i = 0; i < 100; i += 1)
        {
            hidePanel.GetComponent<Image>().color = new Color(0, 0, 0, i / 100);
            yield return new WaitForSeconds(0.002f);
        }

        credits.gameObject.SetActive(true);

        for (float i = 0; i < 100; i += 1)
        {
            hidePanel.GetComponent<Image>().color = new Color(0, 0, 0, 1 - i / 100);
            yield return new WaitForSeconds(0.002f);
        }

        hidePanel.gameObject.SetActive(false);
    }

    bool temp = false;
    IEnumerator CheckForPoints()
    {
        if (!temp)
        {
            temp = true;
            hidePanel.gameObject.SetActive(true);
            for (float i = 0; i < 100; i += 1)
            {
                hidePanel.GetComponent<Image>().color = new Color(0, 0, 0, i / 100);
                yield return new WaitForSeconds(0.002f);
            }

            HighScores _hs = HighScores.Load();
            int index;
            if (_hs.IsScoreBetter(score, out index))
            {
                credits.gameObject.SetActive(false);
                newScorePanel.gameObject.SetActive(true);
                newScorePanel.Initialize(index, score, _hs);
            }
            else
            {
                UnityEngine.SceneManagement.SceneManager.LoadScene("Menu");
            }


            for (float i = 0; i < 100; i += 1)
            {
                hidePanel.GetComponent<Image>().color = new Color(0, 0, 0, 1 - i / 100);
                yield return new WaitForSeconds(0.002f);
            }
        }
        yield return null;
    }

    public void GameOver()
    {
        gamePaused = true;
        StartCoroutine(LooseAnimation());

    }
    IEnumerator LooseAnimation()
    {
        music.Stop();
        gameOverPanel.Activate();
        for (int i = 0; i < 10; i++)
        {
            yield return ExplosionAnimation(new Vector3(7 + Random.Range(-3f, 3f), +Random.Range(-3f, 3f), 0));
        }
        yield return BigExplosionAnimation(new Vector3(7, 0, 0));
        yield return new WaitForSeconds(1f);
    }

    public void AddScore(Enemy e)
    {
        StartCoroutine(ExplosionAnimation(e.transform.position));
        //Score s = Instantiate(scoreUp, e.transform.position, Quaternion.identity);
        //s.Init(e.value);
        score += e.value;
        enemy--;
        UpdateTexts();
    }

    public IEnumerator ExplosionAnimation(Vector3 position)
    {
        explosionSound.Play();
        GameObject explo = Instantiate(explosion, position, Quaternion.Euler(new Vector3(0,0,Random.Range(0,360))));
        yield return new WaitForSeconds(0.25f);
        Destroy(explo);
    }
    public IEnumerator BigExplosionAnimation(Vector3 position)
    {
        explosionSound.Play();
        GameObject explo = Instantiate(bigExplosion, position, Quaternion.Euler(new Vector3(0, 0, Random.Range(0, 360))));
        yield return new WaitForSeconds(1f);
        Destroy(explo);
    }

    void UpdateTexts()
    {
        scoreHUD.text = score.ToString("00000000");
        waveHUD.text = wave.ToString("00");
        enemyHUD.text = enemy.ToString("000");
    }

    public void NextWave(int enemyCount)
    {
        wave++;
        enemy = enemyCount;
        UpdateTexts();
    }

    public void OpenCloseMenu()
    {
        openMenuSound.Play();
        menu.SetActive(!menu.activeSelf);
        Time.timeScale = (menu.activeSelf) ? 0f : 1;
        gamePaused = menu.activeSelf;
    }

    public void Retry()
    {
        OpenCloseMenu();
        UnityEngine.SceneManagement.SceneManager.LoadScene("Game");
    }

    public void Quit()
    {
        OpenCloseMenu();
        StartCoroutine(BackToMenuAnim());
    }
}
