﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class MainMenu : MonoBehaviour {

    [SerializeField] GameObject pressButtonPanel;
    [SerializeField] GameObject mainMenuPanel;
    [SerializeField] GameObject highScorePanel;
    [SerializeField] GameObject beforeGamePanel;
    [SerializeField] AudioSource music;
    [SerializeField] AudioSource validate;
    [SerializeField] AudioSource select;

    [SerializeField] Button[] menuButtons;
    [SerializeField] GameObject cursor;

    [SerializeField] GameObject pressStart;

    [SerializeField] GridLayoutGroup highScoreGroup;
    [SerializeField] Text[] scores;
    int currentIndex = 0;

    Color activeMenuColor = new Color(131f / 255, 255f / 255, 230f / 255);
    Color unactiveMenuColor = new Color(199f / 255, 196f / 255, 182f / 255);

    bool controlsLocked = true;
    bool firstAnimation = true;
    bool interuptedIntro = false;


    bool gamePadUsed = false;

	// Use this for initialization
	void Start () {
        StartCoroutine(StartingScene());
    }

    IEnumerator StartingScene()
    {
        yield return new WaitForSeconds(0.5f);
        for (float i = 0; i < 50; i+=1)
        {
            beforeGamePanel.GetComponent<Image>().color = new Color(i / 50, i / 50, i / 50);
            yield return new WaitForSeconds(0.03f);
            if (interuptedIntro)
            {
                break;
            }
        }
        controlsLocked = false;
        yield return new WaitForSeconds(20f);
        if (!interuptedIntro)
            yield return EndIntro();
    }

    IEnumerator EndIntro()
    {
        controlsLocked = true;
        for (float i = 100; i >= 0; i -= 1)
        {
            beforeGamePanel.GetComponent<Image>().color = new Color(i / 100, i / 100, i / 100);
            yield return new WaitForSeconds(0.001f);
        }
        for (float i = 0; i < 100; i += 1)
        {
            beforeGamePanel.GetComponent<Image>().color = new Color(0, 0, 0, 1f - i / 100);
            yield return new WaitForSeconds(0.001f);
        }
        ChangeIndex(0, false);
        pressStart.SetActive(true);
        beforeGamePanel.SetActive(false);
        controlsLocked = false;
    }
	
	// Update is called once per frame
	void Update ()
    {
        if (controlsLocked) return;
        if (beforeGamePanel.activeSelf && Input.anyKeyDown)
        {
            interuptedIntro = true;
            StartCoroutine(EndIntro());
        }
        else if (highScorePanel.activeSelf && Input.anyKeyDown)
        {
            validate.Play();
            StartCoroutine(ShowHighScore(false));
        }
        else if (pressButtonPanel.activeSelf && Input.anyKeyDown)
        {
            validate.Play();
            pressButtonPanel.SetActive(false);
            mainMenuPanel.SetActive(true);
        }
        else if (mainMenuPanel.activeSelf)
        {

            if (Input.GetKeyDown(KeyCode.S) || Input.GetKeyDown(KeyCode.DownArrow) || (Input.GetAxis("Vertical P1") < 0 && !gamePadUsed))
            {
                ChangeIndex(currentIndex + 1);
                gamePadUsed = true;
            }
            else if (Input.GetKeyDown(KeyCode.W) || Input.GetKeyDown(KeyCode.UpArrow) || (Input.GetAxis("Vertical P1") > 0 && !gamePadUsed))
            {
                ChangeIndex(currentIndex - 1);
                gamePadUsed = true;
            }
            else if (Input.GetButtonDown("Action P1") || Input.GetButtonDown("Action P2"))
            {
                ActiveSelectedMenu();
            }

            if (Input.GetAxis("Vertical P1") == 0)
            {
                gamePadUsed = false;
            }
        }
    }

    void ActiveSelectedMenu()
    {
        validate.Play();
        switch (currentIndex)
        {
            case 0:
                Play();
                break;
            case 1:
                HighScore();
                break;
            case 2:
                Quit();
                break;
        }
    }

    public void ChangeIndex(int idx, bool playSound = true)
    {
        if(playSound)
            select.Play();
        if (idx < 0) idx = 0;
        if (idx >= menuButtons.Length) idx = menuButtons.Length - 1;


        menuButtons[currentIndex].GetComponentInChildren<Text>().color = unactiveMenuColor;
        currentIndex = idx;
        menuButtons[currentIndex].GetComponentInChildren<Text>().color = activeMenuColor;
        cursor.transform.position = new Vector3(cursor.transform.position.x, menuButtons[currentIndex].transform.position.y, 0);
    }


    public void Play()
    {
        StartCoroutine(StartGame());
    }
    IEnumerator StartGame()
    {
        controlsLocked = true;
        // Sound
        beforeGamePanel.SetActive(true);
        for (float i = 0; i < 100; i+= 1)
        {
            music.volume = 1 - i / 100;
            beforeGamePanel.GetComponent<Image>().color = new Color(0,0,0, i/100);
            yield return new WaitForSeconds(0.002f);
        }
        UnityEngine.SceneManagement.SceneManager.LoadScene("Game");
    }

    public void Quit()
    {
        StartCoroutine(QuitAnime());
    }

    IEnumerator QuitAnime()
    {
        beforeGamePanel.SetActive(true);
        for (float i = 0; i < 30; i += 1)
        {
            beforeGamePanel.GetComponent<Image>().color = new Color(0, 0, 0, i / 30);
            yield return new WaitForSeconds(0.002f);
        }
        Application.Quit();

    }

    public void HighScore()
    {
        StartCoroutine(ShowHighScore(true));
    }

    IEnumerator ShowHighScore(bool a)
    {
        controlsLocked = true;
        beforeGamePanel.SetActive(true);
        for (float i = 0; i < 30; i += 1)
        {
            beforeGamePanel.GetComponent<Image>().color = new Color(0, 0, 0, i / 30);
            yield return new WaitForSeconds(0.002f);
        }

        mainMenuPanel.SetActive(!a);
        highScorePanel.SetActive(a);
        UpdateHighScoreList();

        for (float i = 30; i >= 0; i -= 1)
        {
            beforeGamePanel.GetComponent<Image>().color = new Color(0, 0, 0, i / 30);
            yield return new WaitForSeconds(0.002f);
        }
        beforeGamePanel.SetActive(false);
        controlsLocked = false;
    }

    public void UpdateHighScoreList()
    {
        HighScores hs = HighScores.Load();
        for(int i = 0; i < hs.scores.Count; i++)
        {
            scores[i].text = hs.scores[i].tag + " .... "+ hs.scores[i].points.ToString("00000000");
        }
        for (int i = hs.scores.Count; i < 5; i++)
            scores[i].text = "AAA .... 00000000";
    }
}
