﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Turret : Structure {

    [SerializeField] AudioSource[] sounds;
    [SerializeField] AudioSource reloadSound;
    [SerializeField] AudioSource destroySound;

    [SerializeField] Bullet bullet;
    [SerializeField] GameObject canon;
    public int munitionType = 0;
    public int munitionCount = 0;
    public int maxMunitionCount = 0;

    public int shootingInterval = 50;
    int intervalCounter = 0;
    bool shooting = false;
    public PlayerController player;

    public GameObject showAmmo;

    Factory factory;

    // Use this for initialization
    void Start () {

        factory = GameObject.Find("Factory").GetComponent<Factory>();
    }
	
	// Update is called once per frame
	void Update () {
        showAmmo.transform.localScale = new Vector3((float)munitionCount / maxMunitionCount, 1, 1);
        if (intervalCounter > 0)
        {
            intervalCounter--;
        }

    }
    void StartShootingSound()
    {
        switch (munitionType)
        {
            case 1:
                break;
            case 2:
                sounds[0].Play();
                sounds[1].Play();
                break;
        }
    }

    public void StopShootingSound()
    {
        switch (munitionType)
        {
            case 1:
                break;
            case 2:
                sounds[0].Stop();
                sounds[1].Stop();
                sounds[2].Play();
                break;
        }
    }
    public void TurretControl()
    {

        switch (player.playerID)
        {
            case 0:
                if (Input.GetButtonDown("Action P1"))
                {
                    StartShootingSound();
                    shooting = true;
                }
                else if (Input.GetButtonUp("Action P1") && shooting)
                {
                    StopShootingSound();
                    shooting = false;
                }

                if (GameManager.gamePaused) return;

                if (Input.GetButtonDown("Cancel P1"))
                {
                    shooting = false;
                    player.Dismount();
                }
                if (Input.GetKey(KeyCode.S) || Input.GetKey(KeyCode.A) || Input.GetKey(KeyCode.Joystick1Button4)) // L
                {
                    canon.transform.Rotate(new Vector3(0, 0, 1));
                }
                else if (Input.GetKey(KeyCode.D) || Input.GetKey(KeyCode.W) || Input.GetKey(KeyCode.Joystick1Button5)) // R
                {
                    canon.transform.Rotate(new Vector3(0, 0, -1));
                }
                break;
            case 1:
                if (Input.GetButtonDown("Action P2"))
                {
                    StartShootingSound();
                    shooting = true;
                }
                else if (Input.GetButtonUp("Action P2") && shooting)
                {
                    StopShootingSound();
                    shooting = false;
                }

                if (GameManager.gamePaused) return;

                if (Input.GetButtonDown("Cancel P2"))
                {
                    shooting = false;
                    player.Dismount();
                }
                if (Input.GetKey(KeyCode.DownArrow) || Input.GetKey(KeyCode.LeftArrow) || Input.GetKey(KeyCode.Joystick2Button4)) // L
                {
                    canon.transform.Rotate(new Vector3(0, 0, 1));
                }
                else if (Input.GetKey(KeyCode.UpArrow) || Input.GetKey(KeyCode.RightArrow) || Input.GetKey(KeyCode.Joystick2Button5)) // R
                {
                    canon.transform.Rotate(new Vector3(0, 0, -1));
                }
                break;
        }
        if (player != null)
        {
            player.transform.rotation = canon.transform.rotation;

            if (shooting)
                PressShoot();
        }
    }

    protected void PressShoot()
    {
        if (munitionCount > 0 && intervalCounter == 0)
        {
            intervalCounter = shootingInterval;
            Bullet b = Instantiate(bullet, transform.position - canon.transform.right * 1f, canon.transform.rotation, transform);
            munitionCount--;

            if (munitionType == 0)
            {
                sounds[Random.Range(0, 5)].Play();
            } else if(munitionType == 1)
            {
                sounds[0].Play();
                Missile m = (Missile)b;
                m.explodSound = sounds[2];
                m.trajectSound = sounds[1];
            }
        }
        if(munitionCount == 0)
        {
            shooting = false;
            StopShootingSound();
        }
    }

    public void Reload(int qty)
    {
        reloadSound.Play();
        munitionCount += qty;
        munitionCount = Mathf.Min(munitionCount, maxMunitionCount);
    }

    public override void Destroy()
    {
        showAmmo.transform.parent.gameObject.SetActive(false);
        GameManager gm = GameObject.Find("Game Manager").GetComponent<GameManager>();
        gm.StartCoroutine(gm.ExplosionAnimation(transform.position));
        if(player != null)
            player.Dismount();
        Destroy(gameObject);
    }

    void OnTriggerEnter2D(Collider2D col)
    {
        if(col.tag == "Player")
            col.GetComponent<PlayerController>().CanMount(this);
    }
    void OnTriggerExit2D(Collider2D col)
    {
        if(col.tag == "Player")
            col.GetComponent<PlayerController>().CanMount(null);
    }
}
